"""
:mod: `models`
=====================

Description of models within the skill network.

.. module:: skill_models
.. module_author:: Niel (Ketov) Gorev <ketov-x@ya.ru>
.. module_author:: http://t.me/ketov-x
"""

from scintillant.apimodels.models.client_models import ClientResponse
from scintillant.apimodels.models.skill_models import (
    SkillResponse, SkillRequest
)
from scintillant.apimodels.models.dialog_models import DialogAnswerRequest
